/*
* nemitter.js
* @version 0.0.9
* @copyright (c) 2016 - KFlash
* @license MIT <https://github.com/jeibo/nemitter#readme/blob/master/LICENSE>
*/
(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory(require('bluebird')) :
    typeof define === 'function' && define.amd ? define(['bluebird'], factory) :
    (global.nemitter = factory(global.Bluebird));
}(this, function (Bluebird) { 'use strict';

    function defer() {
        var resolve;
        var reject;
        var promise = new Bluebird(function () {
            resolve = arguments[0];
            reject = arguments[1];
        });
        return {
            resolve: resolve,
            reject: reject,
            promise: promise
        };
    }
    ;
    var CAMEL_REGEXP = /[A-Z]/g;
    var camelize = function (camelCase) { return camelCase.replace(CAMEL_REGEXP, function (x, pos) { return (pos > 0 ? '_' : '') + x.toLowerCase(); }); };
    var SLICE_ = Array.prototype.slice;
    function removeFromArray(array, object) {
        if (!array) {
            return null;
        }
        var index = array.indexOf(object);
        var exists = index !== -1;
        if (exists) {
            array.splice(index, 1);
        }
        return exists;
    }
    ;
    var EventEmitter = (function () {
        function EventEmitter() {
            this._events = {};
            this._any = [];
        }
        EventEmitter.prototype._getListeners = function (eventName, createIfMissing) {
            var events = this._getEvents();
            var listeners = events[eventName];
            if (createIfMissing && !listeners) {
                listeners = events[eventName] = [];
            }
            return listeners;
        };
        EventEmitter.prototype._getEvents = function () {
            var events = this._events;
            if (!events) {
                events = this._events = {};
            }
            return events;
        };
        EventEmitter.prototype._getAnyListeners = function (createIfMissing) {
            var any = this._any;
            if (createIfMissing && !any) {
                any = this._any = [];
            }
            return any;
        };
        EventEmitter.prototype.bindAll = function (object, context) {
            context = context || this;
            var bindMethod = function (method) {
                context.addListener(camelize(method.substr(2)), function () {
                    var args = SLICE_.call(arguments, 0);
                    args.push(context);
                    object[method].apply(object, args);
                });
            };
            for (var method in object) {
                if (typeof object[method] === 'function' && method.substr(0, 2) === 'on') {
                    bindMethod(method);
                }
            }
        };
        EventEmitter.prototype.emitAsync = function (event) {
            var pending = this.listeners(event).length;
            var deferred = defer();
            var done = function () {
                if (!--pending) {
                    deferred.resolve();
                }
            };
            this.emit(event, done);
            if (!pending) {
                deferred.resolve();
            }
            return deferred.promise;
        };
        EventEmitter.prototype.emit = function (eventName) {
            var arguments$1 = arguments;

            var _this = this;
            var args = [];
            for (var _i = 1; _i < arguments.length; _i++) {
                args[_i - 1] = arguments$1[_i];
            }
            var emitListeners = function (listeners, type, arg) {
                if (!listeners) {
                    return;
                }
                listeners
                    .slice()
                    .forEach(function (listener, index) {
                    if (listener._once) {
                        if (listener._fired) {
                            return;
                        }
                        listeners.splice(index, 1);
                        listener._fired = true;
                    }
                    if (!!listener.emit) {
                        listener.emit(type, arg);
                    }
                    else {
                        listener.apply(_this, arg);
                    }
                });
            };
            var listeners = this._getListeners(eventName, false);
            // If there is no 'error' event listener then throw.
            if (eventName === 'error' && !listeners) {
                var error = arguments[1];
                if (error instanceof Error) {
                    throw error; // Unhandled 'error' event
                }
                else {
                    // At least give some kind of context to the user
                    var err = new Error('Uncaught, unspecified "error" event. (' + error + ')');
                    ///err.context = error;
                    throw err;
                }
            }
            emitListeners(listeners, eventName, args);
            emitListeners(this._any, eventName, [eventName].concat(args));
        };
        EventEmitter.prototype.on = function (eventName, listener) {
            if (typeof listener !== 'function') {
                throw new TypeError('"listener" argument must be a function');
            }
            // To avoid recursion in the case that eventName === "newListener"! Before
            // adding it to the listeners, first emit "newListener".
            if (this._getListeners('newListener', false)) {
                this.emit('newListener', eventName, listener);
            }
            var listeners = this._getListeners(eventName, true);
            listeners.push(listener);
            return this;
        };
        EventEmitter.prototype.once = function (eventName, listener) {
            if (typeof listener !== 'function') {
                throw new TypeError('"listener" argument must be a function');
            }
            listener._once = true;
            this.on(eventName, listener);
        };
        EventEmitter.prototype.onAny = function (listener) {
            this._getAnyListeners(true).push(listener);
            return this;
        };
        EventEmitter.prototype.offAny = function (listener) {
            removeFromArray(this._getAnyListeners(false), listener);
            return this;
        };
        // emits a 'removeListener' event if the listener was removed
        EventEmitter.prototype.off = function (eventName, listener) {
            if (typeof listener !== 'function') {
                throw new TypeError('"listener" argument must be a function');
            }
            var list = this._getListeners(eventName, false);
            // Nothing to do here...
            if (!list) {
                return this;
            }
            var removed = removeFromArray(list, listener);
            if (removed && this._getEvents().removeListener) {
                this.emit('removeListener', eventName, listener);
            }
            return this;
        };
        EventEmitter.prototype.listeners = function (eventName) {
            var listeners = this._getListeners(eventName, false);
            return listeners ? listeners.slice() : [];
        };
        EventEmitter.prototype.removeAllListeners = function (eventName) {
            var _this = this;
            var events = this._getEvents();
            // nothing to do here...
            if (!events) {
                return this;
            }
            if (!eventName) {
                Object.keys(events).forEach(function (type) { return _this.removeAllListeners(type); });
                return this;
            }
            else {
                var listeners = this._getListeners(eventName, false);
                if (!listeners) {
                    return;
                }
                listeners
                    .slice()
                    .forEach(function (listener) { return _this.off(eventName, listener); });
                delete events[eventName];
            }
            return this;
        };
        EventEmitter.prototype.addListener = function (eventName, listener) { };
        EventEmitter.prototype.removeListener = function (eventName, listener) { };
        EventEmitter.prototype.listenerCount = function (eventName) {
            return this.listeners(eventName).length;
        };
        EventEmitter.listenerCount = function (emitter, eventName) {
            return emitter.listenerCount(eventName);
        };
        return EventEmitter;
    }());
    EventEmitter.prototype.addListener = EventEmitter.prototype.on;
    EventEmitter.prototype.removeListener = EventEmitter.prototype.off;

    return EventEmitter;

}));